@@ -9,7 +9,7 @@ extern crate alloc;
 #[cfg(feature = "alloc")]
 #[cfg(any(
     all(esp32, esp_idf_eth_use_esp32_emac),
-    all(
+    any(
         esp_idf_eth_spi_ethernet_dm9051,
         esp_idf_eth_spi_ethernet_w5500,
         esp_idf_eth_spi_ethernet_ksz8851snl
@@ -38,7 +38,7 @@ pub mod nvs_storage;
     feature = "experimental",
     feature = "alloc",
     esp_idf_comp_app_update_enabled,
-    comp_spi_flash_enabled
+    esp_idf_comp_spi_flash_enabled
 ))]
 pub mod ota;
 pub mod ping;
@@ -46,7 +46,4 @@ pub mod sysloop;
 #[cfg(feature = "alloc")] // TODO: Expose a subset which does not require "alloc"
 pub mod wifi;
 
-#[cfg(any(feature = "binstart", feature = "libstart"))]
-pub mod start;
-
 mod private;
